from haystack import indexes, site
from bambu.pages.models import Page
from datetime import datetime

class PageIndex(indexes.SearchIndex):
	text = indexes.CharField(document = True, use_template = True)
	
	def get_model(self):
		return Page

site.register(Page, PageIndex)


from django.conf.urls import patterns, url
from bambu.pages.views import page
from django.conf import settings

if 'bambu.bootstrap' in settings.INSTALLED_APPS or 'bambu.bootstrap.v2' in settings.INSTALLED_APPS:
	from bambu.bootstrap.v2.decorators import body_classes
elif 'bambu.bootstrap.v3' in settings.INSTALLED_APPS:
	from bambu.bootstrap.v3.decorators import body_classes
else:
	def body_classes(view, *classes):
		return view

urlpatterns = patterns('',
	url(r'^(?P<slug>[\/\w-]+)/$', body_classes(page, 'page'), name = 'page')
)


from django.db import models
from django.contrib.contenttypes import generic
from django.template import Template, Context
from bambu.attachments.models import Attachment
from bambu.pages.managers import PageManager
import logging

class Page(models.Model):
	name = models.CharField(max_length = 100, db_index = True)
	slug = models.SlugField(max_length = 50, db_index = True)
	slug_hierarchical = models.CharField(max_length = 255, unique = True, editable = False)
	order = models.PositiveIntegerField(default = 0, max_length = 2, db_index = True)
	order_hierarchical = models.CharField(max_length = 255, editable = False)
	author = models.ForeignKey('auth.User', related_name = 'pages', editable = False)
	parent = models.ForeignKey('self', related_name = 'children', null = True, blank = True)
	title = models.CharField(max_length = 255, null = True, blank = True)
	subtitle = models.CharField(max_length = 255, null = True, blank = True)
	body = models.TextField(null = True, blank = True)
	css = models.TextField(null = True, blank = True)
	menu_groups = models.CharField(max_length = 255, null = True, blank = True)
	attachments = generic.GenericRelation(Attachment)
	objects = PageManager()
	
	def __unicode__(self):
		return self.name
	
	def render_css(self):
		template = Template(self.css)
		context = Context(
			{
				'attachments': self.attachments.all(),
				'slug': self.slug,
				'pk': self.pk,
				'id': self.pk
			}
		)
		
		return template.render(context)
	
	def update_navigation(self):
		if self.parent:
			slug_parts = self.parent.slug_hierarchical.split('/')
			order_parts = self.parent.order_hierarchical.split('/')
		else:
			slug_parts = []
			order_parts = []
		
		slug_parts.append(self.slug)
		order_parts.append(str(self.order).zfill(2))
		
		self.slug_hierarchical = '/'.join(slug_parts)
		self.order_hierarchical = '/'.join(order_parts)
	
	@models.permalink
	def get_absolute_url(self):
		return ('page', [self.slug_hierarchical])
	
	def get_root_page(self):
		page = self
		parent = self.parent
		
		while parent:
			page = parent
			parent = parent.parent
			if parent is None:
				break
		
		return page
	
	def save(self, *args, **kwargs):
		self.update_navigation()
		super(Page, self).save(*args, **kwargs)
		
		for child in self.children.all():
			child.update_navigation()
			child.save()
	
	class Meta:
		ordering = ('order_hierarchical',)
	
	class QuerySet(models.query.QuerySet):
		def root(self):
			return self.filter(parent__isnull = True)
		
		def menu(self, name):
			return self.filter(menu_groups__icontains = "'%s'" % name)


from bambu.pages.models import Page

def page_or_parent_selected(page, selected):
	if page.pk == selected.pk:
		return True
	
	while True:
		selected = selected.parent
		if selected:
			if page.pk == selected.pk:
				return True
		else:
			break
	
	return False

def page_tree(selected = None, parent = None, show_root = False):
	items = []
	
	if parent and show_root and parent.children.exists():
		if selected and parent.pk == selected.pk:
			is_selected = ' class="active"'
		else:
			is_selected = ''
		
		items.append(
			'<li%s><a href="%s">%s</a>%s</li>' % (
				is_selected,
				parent.get_absolute_url(),
				parent.name,
				page_tree(selected, parent)
			)
		)
	else:
		for page in Page.objects.select_related().filter(parent = parent):
			if selected and page_or_parent_selected(page, selected):
				is_selected = ' class="active"'
			else:
				is_selected = ''
			
			items.append(
				'<li%s><a href="%s">%s</a>%s</li>' % (
					is_selected,
					page.get_absolute_url(),
					page.name,
					page_tree(selected, page)
				)
			)
	
	if len(items) > 0:
		return '<ol class="page-navigation">%s</ol>' % ''.join(items)
	else:
		return ''


# encoding: utf-8
import datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models

class Migration(SchemaMigration):

    def forwards(self, orm):
        
        # Adding model 'Page'
        db.create_table('pages_page', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('name', self.gf('django.db.models.fields.CharField')(max_length=100)),
            ('slug', self.gf('django.db.models.fields.SlugField')(max_length=50, db_index=True)),
            ('slug_hierarchical', self.gf('django.db.models.fields.CharField')(unique=True, max_length=255)),
            ('order', self.gf('django.db.models.fields.PositiveIntegerField')(default=0, max_length=2)),
            ('order_hierarchical', self.gf('django.db.models.fields.CharField')(max_length=255)),
            ('author', self.gf('django.db.models.fields.related.ForeignKey')(related_name='pages', to=orm['auth.User'])),
            ('parent', self.gf('django.db.models.fields.related.ForeignKey')(blank=True, related_name='children', null=True, to=orm['pages.Page'])),
            ('title', self.gf('django.db.models.fields.CharField')(max_length=255, null=True, blank=True)),
            ('subtitle', self.gf('django.db.models.fields.CharField')(max_length=255, null=True, blank=True)),
            ('body', self.gf('django.db.models.fields.TextField')(null=True, blank=True)),
        ))
        db.send_create_signal('pages', ['Page'])


    def backwards(self, orm):
        
        # Deleting model 'Page'
        db.delete_table('pages_page')


    models = {
        'attachments.attachment': {
            'Meta': {'object_name': 'Attachment'},
            'content_type': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['contenttypes.ContentType']"}),
            'description': ('django.db.models.fields.TextField', [], {'null': 'True', 'blank': 'True'}),
            'file': ('django.db.models.fields.files.FileField', [], {'max_length': '100'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'mimetype': ('django.db.models.fields.CharField', [], {'max_length': '50'}),
            'object_id': ('django.db.models.fields.PositiveIntegerField', [], {}),
            'size': ('django.db.models.fields.PositiveIntegerField', [], {}),
            'title': ('django.db.models.fields.CharField', [], {'max_length': '100'})
        },
        'auth.group': {
            'Meta': {'object_name': 'Group'},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '80'}),
            'permissions': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['auth.Permission']", 'symmetrical': 'False', 'blank': 'True'})
        },
        'auth.permission': {
            'Meta': {'ordering': "('content_type__app_label', 'content_type__model', 'codename')", 'unique_together': "(('content_type', 'codename'),)", 'object_name': 'Permission'},
            'codename': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'content_type': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['contenttypes.ContentType']"}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '50'})
        },
        'auth.user': {
            'Meta': {'object_name': 'User'},
            'date_joined': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'email': ('django.db.models.fields.EmailField', [], {'max_length': '75', 'blank': 'True'}),
            'first_name': ('django.db.models.fields.CharField', [], {'max_length': '30', 'blank': 'True'}),
            'groups': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['auth.Group']", 'symmetrical': 'False', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'is_active': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'is_staff': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'is_superuser': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'last_login': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'last_name': ('django.db.models.fields.CharField', [], {'max_length': '30', 'blank': 'True'}),
            'password': ('django.db.models.fields.CharField', [], {'max_length': '128'}),
            'user_permissions': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['auth.Permission']", 'symmetrical': 'False', 'blank': 'True'}),
            'username': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '30'})
        },
        'contenttypes.contenttype': {
            'Meta': {'ordering': "('name',)", 'unique_together': "(('app_label', 'model'),)", 'object_name': 'ContentType', 'db_table': "'django_content_type'"},
            'app_label': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'model': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '100'})
        },
        'pages.page': {
            'Meta': {'ordering': "('order_hierarchical',)", 'object_name': 'Page'},
            'author': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'pages'", 'to': "orm['auth.User']"}),
            'body': ('django.db.models.fields.TextField', [], {'null': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'order': ('django.db.models.fields.PositiveIntegerField', [], {'default': '0', 'max_length': '2'}),
            'order_hierarchical': ('django.db.models.fields.CharField', [], {'max_length': '255'}),
            'parent': ('django.db.models.fields.related.ForeignKey', [], {'blank': 'True', 'related_name': "'children'", 'null': 'True', 'to': "orm['pages.Page']"}),
            'slug': ('django.db.models.fields.SlugField', [], {'max_length': '50', 'db_index': 'True'}),
            'slug_hierarchical': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '255'}),
            'subtitle': ('django.db.models.fields.CharField', [], {'max_length': '255', 'null': 'True', 'blank': 'True'}),
            'title': ('django.db.models.fields.CharField', [], {'max_length': '255', 'null': 'True', 'blank': 'True'})
        }
    }

    complete_apps = ['pages']



from django.contrib import admin
from django.conf import settings
from django import forms
from bambu.pages.models import Page
from bambu.attachments.admin import AttachmentInline

try:
	import json as simplejson
except ImportError:
	from django.utils import simplejson

class PageAdminForm(forms.ModelForm):
	def __init__(self, *args, **kwargs):
		def add_choices(queryset, indent = 0):
			if self.instance:
				queryset = queryset.exclude(pk = self.instance.pk)
			
			for page in queryset:
				choices.append((page.pk, ('-- ' * indent) + page.name))
				add_choices(page.children.all(), indent + 1)
		
		super(PageAdminForm, self).__init__(*args, **kwargs)
		choices = [('', '---------')]
		queryset = self.fields['parent'].queryset
		add_choices(queryset.root())
		
		self.fields['parent'].widget.choices = choices
		
		try:
			from markitup.widgets import MarkItUpWidget
			self.fields['body'].widget = MarkItUpWidget()
		except ImportError:
			pass
		
		self.fields['css'].label = u'Custom CSS'
		self.fields['menu_groups'].help_text = u'A comma-separated name of menu groups'
	
	def save(self, commit = True):
		obj = super(PageAdminForm, self).save(commit = False)
		
		if obj.menu_groups:
			menus = []
			
			for menu in obj.menu_groups.split(','):
				menu = menu.strip()
				
				while menu.startswith('"'):
					menu = menu[1:]
				
				while menu.startswith("'"):
					menu = menu[1:]
				
				while menu.endswith('"'):
					menu = menu[:-1]
				
				while menu.endswith("'"):
					menu = menu[:-1]
				
				menus.append("'%s'" % menu)
			
			obj.menu_groups = ','.join(menus)
		else:
			obj.menu_groups = None
		
		if commit:
			obj.save()
		
		return obj
	
	class Meta:
		model = Page
	
	class Media:
		css = 'bambu.codemirror' in settings.INSTALLED_APPS and {
			'all': ('codemirror/lib/codemirror.css',)
		} or {}
		
		js = 'bambu.codemirror' in settings.INSTALLED_APPS and (
			'codemirror/lib/codemirror.js',
			'codemirror/mode/css/css.js',
			'blog/admin.js'
		) or ()

class PageAdmin(admin.ModelAdmin):
	list_display = ('link_hierarchical', 'parent', 'order_field')
	prepopulated_fields = {
		'slug': ('name',)
	}
	
	inlines = [AttachmentInline]
	form = PageAdminForm
	
	fieldsets = (
		(
			None,
			{
				'fields': ('name', 'slug', 'parent')
			},
		),
		(
			u'Page content',
			{
				'fields': ('title', 'subtitle', 'body', 'css')
			}
		),
		(
			u'Navigation',
			{
				'fields': ('menu_groups', 'order',)
			}
		)
	)
	
	def link_hierarchical(self, obj):
		spaces = 0
		parent = obj.parent
		while parent:
			spaces += 4
			parent = parent.parent
		
		return ('&nbsp;' * spaces) + obj.name
	link_hierarchical.allow_tags = True
	link_hierarchical.short_description = 'Name'
	link_hierarchical.admin_order_field = 'slug_hierarchical'
	
	def order_field(self, obj):
		return obj.order
	order_field.short_description = 'Order'
	order_field.admin_order_field = 'order_hierarchical'
	
	def save_model(self, request, obj, *args, **kwargs):
		obj.author = request.user
		obj.save()

admin.site.register(Page, PageAdmin)


from django.db.models import Manager

class PageManager(Manager):
	def get_query_set(self):
		return self.model.QuerySet(self.model)
	
	def root(self):
		return self.get_query_set().root()
	
	def menu(self, name):
		return self.get_query_set().menu(name)


#!/usr/bin/env python
from distutils.core import setup

setup(
	name = 'bambu-pages',
	version = '0.1',
	description = 'Tools for Django webapps',
	author = 'Steadman',
	author_email = 'mark@steadman.io',
	url = 'http://pypi.python.org/pypi/bambu-pages',
	install_requires = [
		'Django>=1.4',
		'bambu-attachments',
		'bambu-markup',
		'bambu-formatrules'
	],
	namespace_packages = ['bambu'],
	packages = [
		'bambu.pages',
		'bambu.pages.migrations'
	],
	package_data = {
		'bambu.pages': [
			'templates/pages/*.html',
			'templates/search/indexes/pages/*.txt'
		]
	},
	classifiers = [
		'Development Status :: 4 - Beta',
		'Environment :: Web Environment',
		'Framework :: Django'
	]
)


from django.template.response import TemplateResponse
from django.shortcuts import get_object_or_404
from bambu.pages.models import Page
from bambu.pages.helpers import page_tree
from bambu.enqueue import enqueue_css_block

def page(request, slug):
	page = get_object_or_404(Page, slug_hierarchical = slug)
	templates = []
	parent = 'pages/'
	
	title_parts = [page.title or page.name]
	breadcrumb = [('', page.name)]
	backtick = '../'
	parent = page.parent
	styles = [enqueue_css_block(request, page.css)]
	classes = []
	
	while parent:
		templates.append('pages/%s/subpage.html' % parent.slug_hierarchical)
		templates.append(
			'pages/%ssubpage.html' % (
				'subpage/' * len(parent.slug_hierarchical.split('/'))
			)
		)
		
		title_parts.append(parent.title or parent.name)
		breadcrumb.append((backtick, parent.name))
		classes.append('subpage-%s' % parent.slug_hierarchical.replace('-', '_').replace('/', '-'))
		
		styles.append(
			enqueue_css_block(request, parent.css)
		)
		
		parent = parent.parent
		backtick += '../'
	
	breadcrumb.reverse()
	templates.insert(0, 'pages/%s.html' % page.slug_hierarchical)
	
	if page.parent:
		templates.append('pages/subpage.html')
	
	templates.append('pages/page.html')
	classes.append('page-%s' % page.slug_hierarchical.replace('-', '_').replace('/', '-'))
	
	styles.reverse()
	classes.reverse()
	
	siblings = Page.objects.filter(parent = page.parent)
	
	try:
		next = siblings.filter(order__gt = page.order)[0]
	except IndexError:
		next = None
	
	try:
		previous = siblings.filter(order__lt = page.order).order_by('-order')[0]
	except IndexError:
		previous = None
	
	return TemplateResponse(
		request,
		templates,
		{
			'page': page,
			'page_tree': page_tree(page, page.get_root_page(), show_root = True),
			'page_tree_unrooted': page_tree(page, page.get_root_page(), show_root = False),
			'title_parts': title_parts,
			'breadcrumb_trail': breadcrumb,
			'menu_selection': page.get_root_page().slug_hierarchical.replace('/', '-'),
			'enqueued_styles': styles,
			'next_page': next,
			'previous_page': previous,
			'body_classes': classes
		}
	)


from bambu import navigation
from bambu.pages.models import Page

class PageMenuBuilder(navigation.MenuBuilder):
	def register_partials(self):
		yield {
			'name': 'pages',
			'description': 'Lists all the root-level pages in the site'
		}
	
	def add_to_menu(self, name, items, menu_name = None, **kwargs):
		pages = Page.objects.root()
		
		if menu_name:
			pages = pages.menu(menu_name)
		
		for page in pages:
			items.append(
				{
					'url': ('page', [page.slug_hierarchical]),
					'title': page.name,
					'selection': page.get_root_page().slug_hierarchical.replace('/', '-'),
					'order': 10 + page.order
				}
			)

navigation.site.register(PageMenuBuilder)


# -*- coding: utf-8 -*-
import datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models


class Migration(SchemaMigration):

    def forwards(self, orm):
        # Adding index on 'Page', fields ['name']
        db.create_index('pages_page', ['name'])

        # Adding index on 'Page', fields ['order']
        db.create_index('pages_page', ['order'])


    def backwards(self, orm):
        # Removing index on 'Page', fields ['order']
        db.delete_index('pages_page', ['order'])

        # Removing index on 'Page', fields ['name']
        db.delete_index('pages_page', ['name'])


    models = {
        'attachments.attachment': {
            'Meta': {'unique_together': "(('content_type', 'object_id', 'file'),)", 'object_name': 'Attachment'},
            'content_type': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['contenttypes.ContentType']"}),
            'description': ('django.db.models.fields.TextField', [], {'null': 'True', 'blank': 'True'}),
            'featured': ('django.db.models.fields.BooleanField', [], {'default': 'False', 'db_index': 'True'}),
            'file': ('django.db.models.fields.files.FileField', [], {'max_length': '100'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'mimetype': ('django.db.models.fields.CharField', [], {'max_length': '50', 'db_index': 'True'}),
            'object_id': ('django.db.models.fields.PositiveIntegerField', [], {}),
            'size': ('django.db.models.fields.PositiveIntegerField', [], {}),
            'title': ('django.db.models.fields.CharField', [], {'max_length': '100'})
        },
        'auth.group': {
            'Meta': {'object_name': 'Group'},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '80'}),
            'permissions': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['auth.Permission']", 'symmetrical': 'False', 'blank': 'True'})
        },
        'auth.permission': {
            'Meta': {'ordering': "('content_type__app_label', 'content_type__model', 'codename')", 'unique_together': "(('content_type', 'codename'),)", 'object_name': 'Permission'},
            'codename': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'content_type': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['contenttypes.ContentType']"}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '50'})
        },
        'auth.user': {
            'Meta': {'object_name': 'User'},
            'date_joined': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'email': ('django.db.models.fields.EmailField', [], {'max_length': '75', 'blank': 'True'}),
            'first_name': ('django.db.models.fields.CharField', [], {'max_length': '30', 'blank': 'True'}),
            'groups': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['auth.Group']", 'symmetrical': 'False', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'is_active': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'is_staff': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'is_superuser': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'last_login': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'last_name': ('django.db.models.fields.CharField', [], {'max_length': '30', 'blank': 'True'}),
            'password': ('django.db.models.fields.CharField', [], {'max_length': '128'}),
            'user_permissions': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['auth.Permission']", 'symmetrical': 'False', 'blank': 'True'}),
            'username': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '30'})
        },
        'contenttypes.contenttype': {
            'Meta': {'ordering': "('name',)", 'unique_together': "(('app_label', 'model'),)", 'object_name': 'ContentType', 'db_table': "'django_content_type'"},
            'app_label': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'model': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '100'})
        },
        'pages.page': {
            'Meta': {'ordering': "('order_hierarchical',)", 'object_name': 'Page'},
            'author': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'pages'", 'to': "orm['auth.User']"}),
            'body': ('django.db.models.fields.TextField', [], {'null': 'True', 'blank': 'True'}),
            'css': ('django.db.models.fields.TextField', [], {'null': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'menu_groups': ('django.db.models.fields.CharField', [], {'max_length': '255', 'null': 'True', 'blank': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '100', 'db_index': 'True'}),
            'order': ('django.db.models.fields.PositiveIntegerField', [], {'default': '0', 'max_length': '2', 'db_index': 'True'}),
            'order_hierarchical': ('django.db.models.fields.CharField', [], {'max_length': '255'}),
            'parent': ('django.db.models.fields.related.ForeignKey', [], {'blank': 'True', 'related_name': "'children'", 'null': 'True', 'to': "orm['pages.Page']"}),
            'slug': ('django.db.models.fields.SlugField', [], {'max_length': '50'}),
            'slug_hierarchical': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '255'}),
            'subtitle': ('django.db.models.fields.CharField', [], {'max_length': '255', 'null': 'True', 'blank': 'True'}),
            'title': ('django.db.models.fields.CharField', [], {'max_length': '255', 'null': 'True', 'blank': 'True'})
        }
    }

    complete_apps = ['pages']